//------------------------------------------------------------------------------
//
//   Greg's Blocks - TESawbench
//
//------------------------------------------------------------------------------

package gcewing.blocks;

import net.minecraft.src.*;
import net.minecraftforge.common.*;

public class TESawbench extends TileEntity implements IInventory, ISidedInventory {

	final public static int materialSlot = 0;
	final public static int resultSlot = 1;
	
	static int[] materialUsedForShape = new int[] {
		1, 1, 2,  1, 1, 1,  1, 1, 1
	};
	
	static int[] productMadeForShape = new int[] {
		2, 3, 3,  4, 3, 1,  1, 0, 0
	};
	
	public Inventory inventory = new Inventory("Items", 2, this);
	public int selectedShape = 0;
	public boolean pendingMaterialUsage = false; // Material for the stack in the result slot
	                                      // has not yet been removed from the material slot
	
	@Override
	public int getSizeInventory() {
		return inventory.getSizeInventory();
	}
	
	@Override
	public ItemStack getStackInSlot(int i) {
		//if (!worldObj.isRemote)
		//	System.out.printf("TESawbench.getStackInSlot: %d on %s\n", i, worldObj);
		return inventory.getStackInSlot(i);
	}
	
	@Override
	public ItemStack getStackInSlotOnClosing(int i) {
		return null; // Leave items in inventory on closing
		//return inventory.getStackInSlotOnClosing(i);
	}

	@Override
	public void setInventorySlotContents(int i, ItemStack stack) {
		//System.out.printf("TESawbench.setInventorySlotContents: %d to %s on %s\n", i, stack, worldObj);
		beforeChangeSlot(i);
		inventory.setInventorySlotContents(i, stack);
		updateResultSlot();
	}
	
	@Override
	public ItemStack decrStackSize(int slot, int amount) {
		//System.out.printf("TESawbench.decrStackSize: %d by %d on %s\n", slot, amount, worldObj);
		beforeChangeSlot(slot);
		ItemStack result = inventory.decrStackSize(slot, amount);
		updateResultSlot();
		return result;
	}
	
	void beforeChangeSlot(int slot) {
		//System.out.printf("TESawbench.beforeChangeSlot: %d pmu = %s\n", slot, pendingMaterialUsage);
		if (slot == resultSlot && pendingMaterialUsage) {
			//System.out.printf("TESawbench.beforeChangeSlot: setting pmu to false\n");
			pendingMaterialUsage = false;
			ItemStack result = getStackInSlot(resultSlot);
			if (result != null) {
				//System.out.printf("TESawbench.beforeChangeSlot: using material\n");
				inventory.decrStackSize(materialSlot, materialMultiple());
			}
		}
	}
	
	@Override
	public String getInvName() {
		return inventory.getInvName();
	}
	
//	public void onInventoryChanged() { 
//		System.out.printf("TESawbench.onInventoryChanged: %s\n", worldObj);
//		super.onInventoryChanged();
//	}

	/**
	 * Get the start of the side inventory.
	 * @param side The global side to get the start of range.
	 *		0: -Y (bottom side)
	 *		1: +Y (top side)
	 *		2: -Z
	 *		3: +Z
	 *		4: -X
	 *		5: +x
	 */
	@Override
	public int getStartInventorySide(ForgeDirection side) {
		if (side == ForgeDirection.UP)
			return materialSlot;
		else
			return resultSlot;
	}

	/**
	 * Get the size of the side inventory.
	 * @param side The global side.
	 */
	@Override
	public int getSizeInventorySide(ForgeDirection side) {
		return 1;
	}

	@Override
	public void readFromNBT(NBTTagCompound tc) {
		super.readFromNBT(tc);
		inventory.readFromNBT(tc);
		selectedShape = tc.getInteger("Shape");
		pendingMaterialUsage = tc.getBoolean("PMU");
	}

	@Override
	public void writeToNBT(NBTTagCompound tc) {
		super.writeToNBT(tc);
		inventory.writeToNBT(tc);
		tc.setInteger("Shape", selectedShape);
		tc.setBoolean("PMU", pendingMaterialUsage);
	}

	@Override
	public int getInventoryStackLimit() {
		return inventory.getInventoryStackLimit();
	}

	@Override
	public boolean isUseableByPlayer(EntityPlayer player) {
		if (worldObj.getBlockTileEntity(xCoord, yCoord, zCoord) == this)
			return player.getDistanceSq(xCoord + 0.5, yCoord + 0.5, zCoord + 0.5) <= 64.0;
		else
			return false;
	}

	@Override
	public void openChest() {
		inventory.openChest();
	}

	@Override
	public void closeChest() {
		inventory.closeChest();
	}
	
	public void setSelectedShape(int i) {
		if (i >= 0 && i < TEShape.numShapes) {
			selectedShape = i;
			updateResultSlot();
		}
	}
	
	void updateResultSlot() {
		//System.out.printf("TESawbench.updateResultSlot: pmu = %s on %s\n",
		//	pendingMaterialUsage, worldObj);
		ItemStack oldResult = getStackInSlot(resultSlot);
		if (oldResult == null || pendingMaterialUsage) {
			ItemStack materialStack = getStackInSlot(materialSlot);
			ItemStack resultStack = null;
			//System.out.printf("TESawbench.updateResultSlot: material = %s\n", materialStack);
			//System.out.printf("TESawbench.updateResultSlot: old result = %s\n", resultStack);
			if (materialStack != null && materialStack.stackSize >= materialMultiple()) {
				Item materialItem = materialStack.getItem();
				if (materialItem instanceof ItemBlock) {
					Block materialBlock = Block.blocksList[((ItemBlock)materialItem).getBlockID()];
					//System.out.printf("TESawbench.updateResultSlot: material block id = %d\n",
					//	materialBlock.blockID);
					if (materialBlock.isOpaqueCube()) {
						TEShape resultTE = new TEShape(selectedShape, materialBlock.blockID, materialStack.getItemDamage());
						resultStack = Utils.blockStackWithTileEntity(GregsBlocks.blockShape, resultMultiple(), resultTE);
					}
				}
			}
			if (!ItemStack.areItemStacksEqual(resultStack, oldResult)) {
				//System.out.printf("TESawbench.updateResultSlot: setting result to %s on %s\n",
				//	resultStack, worldObj);
				inventory.setInventorySlotContents(resultSlot, resultStack);
			}
			pendingMaterialUsage = resultStack != null;
			//System.out.printf("TESawbench.updateResultSlot: now pmu = %s on %s\n",
			//	pendingMaterialUsage, worldObj);
		}
	}
	
	int materialMultiple() {
		return materialUsedForShape[selectedShape];
	}
	
	int resultMultiple() {
		return productMadeForShape[selectedShape];
	}
	
}
